@@ -1,11 +1,12 @@
 /** \file mainwindow.cpp */
 
-// Copyright (C) 2018-2022 Sanford Rockowitz <rockowitz@minsoft.com>
+// Copyright (C) 2018-2024 Sanford Rockowitz <rockowitz@minsoft.com>
 // SPDX-License-Identifier: GPL-2.0-or-later
 
 #include "main/mainwindow.h"
 
 #include <assert.h>
+#include <syslog.h>
 #include <iostream>
 
 #include <QtCore/QList>
@@ -17,24 +18,31 @@
 #include <QtWidgets/QShortcut>     // 5.9
 
 #include <ddcutil_c_api.h>
+#include <ddcutil_status_codes.h>
 
-#include "base/core.h"
-#include "base/global_state.h"
-#include "base/widget_debug.h"
-#include "help/help_dialog.h"
-#include "help/help_browser.h"
-
+#include "base/ddcui_core_aux.h"
+#include "base/ddcui_core.h"
 #include "base/ddcui_parms.h"
 #include "base/global_state.h"
+#include "base/global_state.h"
 #include "base/monitor.h"
-#include "core_widgets/spin_slider.h"
+#include "base/nc_values_state.h"
+#include "base/user_interface_options_state.h"
+#include "base/widget_debug.h"
+
+#include "help/help_browser.h"
+#include "help/help_dialog.h"
 
 #include "nongui/msgbox_queue.h"
 #include "nongui/vcpthread.h"    // includes vcprequest.h
 
+#include "cmdline/ddcui_parsed_cmd.h"
+
 #include "monitor_desc/monitor_desc_actions.h"
 #include "monitor_desc/monitor_desc_ui.h"
 
+#include "core_widgets/spin_slider.h"
+
 #include "feature_value_widgets/value_stacked_widget.h"
 
 #include "feature_scrollarea/feature_widget.h"
@@ -44,6 +52,7 @@
 
 #include "option_dialogs/feature_selection_dialog.h"
 #include "option_dialogs/user_interface_options_dialog.h"
+#include "option_dialogs/nc_values_dialog.h"
 
 #include "action_dialogs/debug_actions_dialog.h"
 
@@ -51,11 +60,80 @@
 #include "main/msgbox_thread.h"
 #include "main/mainwindow.h"
 
-#include "../base/nc_values_state.h"
-#include "../option_dialogs/nc_values_dialog.h"
 
 using namespace std;
 
+
+intmax_t get_thread_id2() {
+   pid_t tid = syscall(SYS_gettid);
+   return tid;
+}
+
+
+void create_timestamp2(char* buf, int bufsz) {
+   assert(bufsz >= 40);
+   time_t epoch_seconds = time(NULL);
+   struct tm broken_down_time;
+   localtime_r(&epoch_seconds, &broken_down_time);
+   strftime(buf, 40, "%b %d %T", &broken_down_time);
+}
+
+
+
+void display_status_event_main_callback(DDCA_Display_Status_Event evt) {
+   char time_buf[40];
+   create_timestamp2(time_buf, 40);
+   intmax_t thread_id = get_thread_id2();
+  // printf("(%s) evt.dref=%p event_type=%d\n", __func__, evt.dref, evt.event_type);
+
+  printf("[%s][%6jd](mainwindow.cpp/%s) Executing. dref=%s, event_type = %s\n",
+        time_buf, thread_id, __func__, ddca_dref_repr(evt.dref), ddca_display_event_type_name(evt.event_type));
+
+  printf("[%s][%6jd](mainwindow.cpp/%s) ddca_validate_display_ref(%s) reports: %s\n",
+        time_buf, thread_id, __func__,
+        ddca_dref_repr(evt.dref),
+        ddca_rc_name(ddca_validate_display_ref(evt.dref, true)));
+
+#ifdef REF
+  QString qsTitle = QString("ddcutil Error");
+  QString qsDetail = QString("Invalid Model: %1").arg(parsed_cmd->model);
+  QMessageBox::Icon icon = QMessageBox::Warning;
+  MsgBoxQueueEntry * qe = new MsgBoxQueueEntry(qsTitle, qsDetail, icon);
+#ifdef DEFERRED_MSG_QUEUE
+  _deferredMsgs.append(qe);     // not needed
+#endif
+  TRACECF(debug, "Pre put, _msgBoxQueue=%p", _msgBoxQueue);
+  _msgBoxQueue->put(qe);
+#endif
+
+
+  if (evt.event_type == DDCA_EVENT_DISPLAY_CONNECTED || evt.event_type == DDCA_EVENT_DISPLAY_DISCONNECTED) {
+     QString qstitle("Display Status Change");
+     QMessageBox::Icon icon = QMessageBox::Warning;
+     QString qstext;;
+     // QString qstext = (evt.event_type == DDCA_EVENT_DISPLAY_CONNECTED)
+     //                    ?   QString("Display has been connected.   Redetect Displays")
+     //                    :   QString("Display has been disconnected.  Redetect displays");
+     if (evt.event_type == DDCA_EVENT_DISPLAY_CONNECTED)
+        qstext = QString("Display connected on %1, bus /dev/i2c-%2.\n\nRedetect displays")
+                        .arg(evt.connector_name).arg(  evt.io_path.path.i2c_busno) ;
+      else {
+        qstext = QString("Display disconnected on %1, bus /dev/i2c-%2.\n\nRedetect displays")
+                        .arg(evt.connector_name).arg(evt.io_path.path.i2c_busno);
+      }
+
+     MsgBoxQueueEntry* qe = new MsgBoxQueueEntry(qstitle, qstext, icon);
+     GlobalState::instance()._msgBoxQueue->put(qe);
+     // GlobalState::instance()._mainWindow->on_actionRedetect_triggered();  // creates new window
+
+  }
+  else {
+     printf("[%s][%6jd](mainwindow.cpp/%s) Ignoring event of type %s\n",
+           time_buf, thread_id, __func__, ddca_display_event_type_name(evt.event_type));
+  }
+}
+
+
 //
 // Constructor, Destructor, Initialization
 //
@@ -419,6 +497,26 @@ MainWindow::MainWindow(Parsed_Ddcui_Cmd * parsed_cmd, QWidget *parent) :
     globalState._msgBoxThread = _msgBoxThread;
     globalState._msgBoxQueue  = _msgBoxQueue;
 
+    if (parsed_cmd->flags & CMD_FLAG_WATCH_DISPLAYS) {
+       DDCA_Display_Event_Class event_classes;
+       DDCA_Status watch_rc = ddca_get_active_watch_classes(&event_classes);
+       if (watch_rc  == DDCRC_OK) {
+          if (!(event_classes & DDCA_EVENT_CLASS_DISPLAY_CONNECTION)) {
+             ddca_stop_watch_displays(true);
+             ddca_start_watch_displays(DDCA_EVENT_CLASS_DISPLAY_CONNECTION);
+             if (test_emit_ddcui_syslog(DDCA_SYSLOG_NOTICE))
+                syslog(LOG_NOTICE, "Restarted display watch thread with DDCA_EVENT_CLASS_DISPLAY_CONNECTION");
+          }
+       }
+       else {
+          DDCA_Status rc = ddca_start_watch_displays(DDCA_EVENT_CLASS_DISPLAY_CONNECTION);
+          assert(rc == DDCRC_OK);
+          if (test_emit_ddcui_syslog(DDCA_SYSLOG_NOTICE))
+             syslog(LOG_NOTICE, "Started display watch thread with DDCA_EVENT_CLASS_DISPLAY_CONNECTION");
+       }
+       ddca_register_display_status_callback(display_status_event_main_callback);
+    }
+
      // QShortcut * quit_shortcut = new QShortcut(QKeySequence(Qt::Key_Q | Qt::CTRL), this, SLOT(close()));
      _quit_shortcut = new QShortcut(QKeySequence(Qt::Key_Q | Qt::CTRL), this);
      _quit_shortcut->setContext(Qt::ApplicationShortcut);
@@ -936,6 +1034,27 @@ void MainWindow::on_actionRedetect_triggered() {
 }
 
 
+void MainWindow::on_actionDebugLocks_triggered() {
+   bool debug = true;
+   TRACECF(debug, "Executing");
+   ddca_report_locks(0);
+   captureLocks();
+   TRACECF(debug,"Done");
+}
+
+
+void MainWindow::captureLocks() {
+       ddca_start_capture(DDCA_CAPTURE_NOOPTS);
+       // DDCA_Output_Level saved_ol = ddca_get_output_level();
+       // ddca_set_output_level(DDCA_OL_VERBOSE);
+       ddca_report_locks(0);
+       // ddca_set_output_level(saved_ol);
+       char * s = ddca_end_capture();
+       QString qs(s);
+       free(s);
+       showCapturedText("Execution Statistics", qs);
+}
+
 // Actions->Debug: DebugActionsDialog slots
 
 void MainWindow::on_actionDebugActionsDialog_triggered()
@@ -973,15 +1092,30 @@ void MainWindow::for_reportStats_triggered(DDCA_Stats_Type stats_type, bool show
 }
 
 
+void MainWindow::showCapturedText(QString windowTitle, QString text) {
+   const QFont& textFont = QFont(       "Monospace",  9, QFont::Normal);
+   // viewHelpByTextX(qs, QString("Statistics Report"), textFont,  this);
+   HelpDialog* hd = new HelpDialog(this);
+   hd->setFont(textFont);
+   hd->setText(text);
+   hd->resize(650,40);
+      // hd->_textBrowser->setSource(fn);
+   hd->setWindowTitle( windowTitle );
+   hd->exec();     // always modal
+}
+
+
 void MainWindow::capture_stats(DDCA_Stats_Type stats_type, bool show_thread_data) {
        ddca_start_capture(DDCA_CAPTURE_NOOPTS);
        // DDCA_Output_Level saved_ol = ddca_get_output_level();
        // ddca_set_output_level(DDCA_OL_VERBOSE);
        ddca_show_stats(stats_type, show_thread_data, 0);
        // ddca_set_output_level(saved_ol);
-       char * s = ddca_end_capture();  // API's buffer, do not free
+       char * s = ddca_end_capture();
        QString qs(s);
        free(s);
+       showCapturedText("Execution Statistics", qs);
+#ifdef MOVED
        const QFont& textFont = QFont(       "Monospace",  9, QFont::Normal);
        // viewHelpByTextX(qs, QString("Statistics Report"), textFont,  this);
        HelpDialog* hd = new HelpDialog(this);
@@ -991,6 +1125,7 @@ void MainWindow::capture_stats(DDCA_Stats_Type stats_type, bool show_thread_data
           // hd->_textBrowser->setSource(fn);
        hd->setWindowTitle( "Execution Statistics" );
        hd->exec();     // always modal
+#endif
 }
 
 
@@ -1171,11 +1306,12 @@ void MainWindow::on_actionAbout_triggered()
     // QString ans1 = (build_opts & DDCA_BUILT_WITH_ADL) ? "true" : "false";
     QString ans2 = (build_opts & DDCA_BUILT_WITH_USB) ? "true" : "false";
     QString ans3 = (ddca_is_udf_enabled()) ? "true" : "false";
+    QString libname = ddca_libddcutil_filename();
 
     // QMessageBox mbox;
     // mbox.setText("About ddcutil");
 
-    QString copyright = "Copyright (C) 2014-2022 Sanford Rockowitz\n";
+    QString copyright = "Copyright (C) 2014-2023 Sanford Rockowitz\n";
     copyright = copyright + "Licensed under the terms of the GNU General Public License (GPL) ";
     copyright = copyright + "Version 2.0 or later.";
 
@@ -1183,6 +1319,7 @@ void MainWindow::on_actionAbout_triggered()
     QString msg = "";
     msg = msg + "ddcui version:    " + ddcui_version   + "\n";
     msg = msg + "   User defined features enabled:      " + ans3 + "\n\n";
+    msg = msg + "Using shared library:\n   " + libname + "\n\n";
     msg = msg + "ddcutil version:  " + ddcutil_version + "\n";
  // msg = msg + "   Built with support for ADL connected monitors: " + ans1 + "\n";
     msg = msg + "   Built with support for USB connected monitors: " + ans2;
